from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns
from .views import AccountCreate, AccountDetails, PasswordReset, PasswordUpdate, ObtainAuthToken

urlpatterns = {
    url(r'^account/create$', AccountCreate.as_view(), name='lp_accounts_create'),
    url(r'^account$', AccountDetails.as_view(), name='lp_accounts_retrieve'),
    url(r'^account/password/reset', PasswordReset.as_view(), name='password_reset'),
    url(r'^account/password/update', PasswordUpdate.as_view(), name='password_update'),
    url(r'^login/', ObtainAuthToken.as_view(), name='login')
}

urlpatterns = format_suffix_patterns(urlpatterns)



from django.core import mail
from rest_framework import generics, permissions, status
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from .serializers import UserSerializer, PasswordResetSerializer, PasswordUpdateSerializer, AuthTokenSerializer
from .models import User
from rest_condition import Or
from .permissions import IsAnonUser


class AccountCreate(generics.CreateAPIView):
    """
    post: Create new user
    """
    model = User
    serializer_class = UserSerializer
    permission_classes = [
        Or(IsAnonUser, permissions.IsAdminUser)
    ]

    def post(self, request):
        response = super(AccountCreate, self).post(request)
        token, created = Token.objects.get_or_create(user_id=response.data['id'])
        return Response({'token': token.key, 'id': token.user_id}, status=status.HTTP_201_CREATED)


class AccountDetails(generics.RetrieveAPIView):
    """
    get: Retrieve user details
    """
    model = User
    serializer_class = UserSerializer
    queryset = User.objects.all()
    permission_classes = [
        permissions.IsAuthenticated
    ]

    def get_object(self):
        return self.request.user


class PasswordReset(generics.GenericAPIView):
    """
    post: Send password reset email
    """
    serializer_class = PasswordResetSerializer
    permission_classes = {
        IsAnonUser
    }

    def post(self, request):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        token, created = Token.objects.get_or_create(user=serializer.user)
        mail.send_mail(
            'Password Reset Request', 'This is the link: ' + token.key,
            'support@quadco.com', [request.data['email']],
            fail_silently=False,
        )

        return Response(status=status.HTTP_204_NO_CONTENT)


class PasswordUpdate(generics.GenericAPIView):
    """
    post: Update user password
    """
    model = User
    serializer_class = PasswordUpdateSerializer
    queryset = User.objects.all()
    permission_classes = [
        IsAnonUser
    ]

    def post(self, request):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        serializer.user.set_password(request.data['password'])
        serializer.user.save()

        return Response(status=status.HTTP_200_OK)


class ObtainAuthToken(ObtainAuthToken):
    serializer_class = AuthTokenSerializer
    permission_classes = [
        IsAnonUser
    ]



from django.db import models
from django.contrib.auth.models import AbstractUser


# Create your models here.
class User(AbstractUser):
    TYPE_DEFAULT = 1
    TYPE_GOOGLE = 2
    TYPE_FACEBOOK = 3
    TYPES = (
        (TYPE_DEFAULT, 'Default'),
        (TYPE_GOOGLE, 'Google'),
        (TYPE_FACEBOOK, 'Facebook')
    )

    type = models.IntegerField(choices=TYPES, default=TYPE_DEFAULT)

    def __str__(self):
        return self.username



from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from rest_framework.authtoken.serializers import AuthTokenSerializer
from rest_framework.authtoken.models import Token
from rest_framework.validators import UniqueValidator
from .models import User
from .validators import get_validator

import django.contrib.auth.password_validation as validators
import random
import string


class UserSerializer(serializers.ModelSerializer):
    password = serializers.CharField(write_only=True, required=False)
    username = serializers.CharField(required=True, validators=[
        UniqueValidator(queryset=User.objects.all())
    ])
    email = serializers.CharField(required=True, validators=[
        UniqueValidator(queryset=User.objects.all())
    ])
    type = serializers.ChoiceField(choices=User.TYPES)
    access_token = serializers.CharField(label=_('Access Token'), required=False)
    is_active = serializers.HiddenField(default=True)

    class Meta:
        model = User
        fields = '__all__'

    def create(self, validated_data):
        user = User.objects.create(username=validated_data['username'], email=validated_data['email'])
        user.set_password(validated_data['password'])
        user.save()
        return user

    def validate(self, attrs):
        account_type = attrs.get('type')
        access_token = attrs.get('access_token')
        password = attrs.get('password')

        if account_type is not User.TYPE_DEFAULT and access_token:
            validator = get_validator(attrs['type'])
            if validator.validate(access_token):
                attrs['password'] = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(16))
            else:
                raise serializers.ValidationError('access_token is not valid')
        elif account_type is not User.TYPE_DEFAULT:
            raise serializers.ValidationError('access_token is required')

        if account_type is User.TYPE_DEFAULT and password:
            validators.validate_password(password)
        elif account_type is User.TYPE_DEFAULT:
            raise serializers.ValidationError('password is required')

        return super(UserSerializer, self).validate(attrs)


class PasswordResetSerializer(serializers.Serializer):
    email = serializers.EmailField()

    def validate_email(self, value):
        try:
            self.user = User.objects.get(email=value)
        except ObjectDoesNotExist as e:
            raise serializers.ValidationError(e)

        return value


class PasswordUpdateSerializer(serializers.Serializer):
    token = serializers.CharField()
    password = serializers.CharField()

    def validate_token(self, value):
        try:
            self.token = Token.objects.get(key=value)
            self.user = User.objects.get(id=self.token.user_id)
        except ObjectDoesNotExist as e:
            raise serializers.ValidationError(e)

        return value


class AuthTokenSerializer(AuthTokenSerializer):
    account_type = serializers.ChoiceField(label=_('Account Type'), choices=User.TYPES)
    access_token = serializers.CharField(label=_('Access Token'), required=False)
    password = serializers.CharField(label=_("Password"), style={'input_type': 'password'}, required=False)

    def validate(self, attrs):
        username = attrs.get('username')
        account_type = attrs.get('account_type')
        access_token = attrs.get('access_token')

        if account_type is not User.TYPE_DEFAULT and access_token:
            try:
                validator = get_validator(account_type)
            except ValidationError as e:
                raise serializers.ValidationError(e)

            if validator.validate(access_token):
                try:
                    attrs['user'] = User.objects.get(username=username)
                except ObjectDoesNotExist as e:
                    raise serializers.ValidationError('user with username ' + username + ' not found')
            else:
                raise serializers.ValidationError('access_token is not valid')
        elif account_type is not User.TYPE_DEFAULT:
            raise serializers.ValidationError('username, account_type, and access_token are required for social login')

        if account_type is User.TYPE_DEFAULT:
            attrs = super(AuthTokenSerializer, self).validate(attrs)

        return attrs



from django.conf import settings
import sys

# Flag for Testing Environment
settings.TESTING = len(sys.argv) > 1 and sys.argv[1] == 'test'

# Extend Native User Model
settings.AUTH_USER_MODEL = 'lp_accounts.User'

# REST Framework Configuration
settings.REST_FRAMEWORK = {
   'DEFAULT_AUTHENTICATION_CLASSES': (
       'rest_framework.authentication.TokenAuthentication',
   )
}

# Google Sign-In Integration
# https://developers.google.com/identity/sign-in/web/backend-auth
settings.GOOGLE_APP_ID = getattr(settings, 'GOOGLE_APP_ID', None)
settings.GOOGLE_VALIDATION_URL = getattr(settings, 'GOOGLE_VALIDATION_URL', 'https://www.googleapis.com/oauth2/v3/tokeninfo?access_token=%s')

# Facebook Login Integration
# https://developers.facebook.com/docs/facebook-login/manually-build-a-login-flow#checktoken
settings.FACEBOOK_APP_ID = getattr(settings, 'FACEBOOK_APP_ID', None)
settings.FACEBOOK_CLIENT_SECRET = getattr(settings, 'FACEBOOK_CLIENT_SECRET', None)
settings.FACEBOOK_VALIDATION_URL = getattr(settings, 'FACEBOOK_VALIDATION_URL', 'https://graph.facebook.com/debug_token?input_token=%s&access_token=%s|%s')


from django.contrib import admin
from .models import User
from django.contrib.auth.admin import UserAdmin

UserAdmin.fieldsets += (
    ('Additional Data', {'fields': ('type',)}),
)
admin.site.register(User, UserAdmin)



from django.contrib.auth import get_user_model
from django.core import mail
from django.core.urlresolvers import reverse
from django.test import TestCase, override_settings
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient
from parameterized import parameterized
from .models import User

User = get_user_model()


class AccountTestCase(TestCase):
    fixtures = ['users']

    def setUp(self):
        self.client = APIClient()

    def authenticate(self, user=None):
        self.client.credentials()
        if user:
            token, created = Token.objects.get_or_create(user=user)
            self.client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)

    def create(self, **kwargs):
        return self.client.post(
            reverse('lp_accounts_create'),
            format='json',
            **kwargs
        )

    def login(self, **kwargs):
        return self.client.post(
            reverse('login'),
            format='json',
            **kwargs
        )

    def retrieve(self, **kwargs):
        return self.client.get(
            reverse('lp_accounts_retrieve'),
            format='json',
            **kwargs
        )

    def reset(self, **kwargs):
        return self.client.post(
            reverse('password_reset'),
            format='json',
            **kwargs
        )

    def update(self, **kwargs):
        return self.client.post(
            reverse('password_update'),
            format='json',
            **kwargs
        )

    @staticmethod
    def modify_data(data={}, exclude=[], injections={}):
        data = {**data, **injections}
        for field in exclude:
            data.pop(field, None)

        return data

    @staticmethod
    def get_admin_user():
        return User.objects.get(username='ken.griffey')

    @staticmethod
    def get_user_type_default():
        return User.objects.get(username='tino.martinez')

    @staticmethod
    def get_user_type_google():
        return User.objects.get(username='derek.jeter')

    @staticmethod
    def get_user_type_facebook():
        return User.objects.get(username='mo.vaughn')

    @staticmethod
    def get_user_type_default_data():
        return {
            'username': 'test@launchpeer.com',
            'password': '5qVrlRQzxvHJViRUi4TXek6MxWu18MfKi012u40CFiI=',
            'first_name': 'Launch',
            'last_name': 'Monkey',
            'email': 'launchmonkey@launchpeer.com',
            'type': User.TYPE_DEFAULT,
        }

    @staticmethod
    def get_user_type_google_data():
        return {
            'username': 'launchmonkey_google@launchpeer.com',
            'access_token': 'thisisanaccesstoken',
            'first_name': 'Launch',
            'last_name': 'Monkey',
            'email': 'launchmonkey_google@launchpeer.com',
            'type': User.TYPE_GOOGLE,
        }

    @staticmethod
    def get_user_type_facebook_data():
        return {
            'username': 'launchmonkey_facebook@launchpeer.com',
            'access_token': 'thisisanaccesstoken',
            'first_name': 'Launch',
            'last_name': 'Monkey',
            'email': 'launchmonkey_facebook@launchpeer.com',
            'type': User.TYPE_FACEBOOK,
        }


class AccountCreateTestCase(AccountTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_201_CREATED),
        ('Authenticated User', AccountTestCase.get_user_type_default(), status.HTTP_403_FORBIDDEN),
        ('Authenticated Admin User', AccountTestCase.get_admin_user(), status.HTTP_201_CREATED)
    ]
    REQUIRED_FIELDS = [
        ('username', 'username'),
        ('email', 'email'),
        ('password', 'password'),
        ('type', 'type')
    ]
    REQUIRED_FIELDS_GOOGLE = [
        ('username', 'username'),
        ('email', 'email'),
        ('type', 'type'),
        ('access_token', 'access_token'),
    ]
    REQUIRED_FIELDS_FACEBOOK = [
        ('username', 'username'),
        ('email', 'email'),
        ('type', 'type'),
        ('access_token', 'access_token'),
    ]

    @parameterized.expand(PERMISSIONS)
    def test_default_account_create_permissions(self, _, user, expected):
        self.authenticate(user)
        response = self.create(data=AccountTestCase.get_user_type_default_data())
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(PERMISSIONS)
    @override_settings(GOOGLE_APP_ID='testappid')
    def test_google_account_create_permissions(self, _, user, expected):
        self.authenticate(user)
        response = self.create(data=AccountTestCase.get_user_type_google_data())
        self.assertEqual(response.status_code, expected, response.data)

    @parameterized.expand(PERMISSIONS)
    @override_settings(FACEBOOK_APP_ID='testappid', FACEBOOK_CLIENT_SECRET='testclientsecret')
    def test_facebook_account_create_permissions(self, _, user, expected):
        self.authenticate(user)
        response = self.create(data=AccountTestCase.get_user_type_facebook_data())
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(REQUIRED_FIELDS)
    def test_required_fields_default_account_create(self, _, field):
        self.authenticate(user=None)
        data = self.modify_data(data=AccountTestCase.get_user_type_default_data(), exclude=[field])
        response = self.create(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @parameterized.expand(REQUIRED_FIELDS_GOOGLE)
    @override_settings(GOOGLE_APP_ID='testappid')
    def test_required_fields_google_account_create(self, _, field):
        self.authenticate(user=None)
        data = self.modify_data(data=AccountTestCase.get_user_type_google_data(), exclude=[field])
        response = self.create(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @parameterized.expand(REQUIRED_FIELDS_FACEBOOK)
    @override_settings(FACEBOOK_APP_ID='testappid', FACEBOOK_CLIENT_SECRET='testclientsecret')
    def test_required_fields_facebook_account_create(self, _, field):
        self.authenticate(user=None)
        data = self.modify_data(data=AccountTestCase.get_user_type_facebook_data(), exclude=[field])
        response = self.create(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_validates_type(self):
        self.authenticate(user=None)
        data = self.modify_data(data=AccountTestCase.get_user_type_default_data(), injections={'type': 100})
        response = self.create(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_password_is_secure(self):
        self.authenticate(user=None)
        data = self.modify_data(data=AccountTestCase.get_user_type_default_data(), injections={'password': 'password'})
        response = self.create(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_google_signin_config_check(self):
        self.authenticate(user=None)
        response = self.create(data=self.get_user_type_google_data())
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_facebook_login_config_check(self):
        self.authenticate(user=None)
        response = self.create(data=self.get_user_type_facebook_data())
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class DefaultAccountLoginTestCase(AccountTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_200_OK),
        ('Authenticated User', AccountTestCase.get_user_type_default(), status.HTTP_403_FORBIDDEN),
        ('Authenticated Admin User', AccountTestCase.get_admin_user(), status.HTTP_403_FORBIDDEN)
    ]
    REQUIRED_FIELDS = [
        ('username', 'username'),
        ('password', 'password'),
        ('account_type', 'account_type')
    ]

    def setUp(self):
        super(DefaultAccountLoginTestCase, self).setUp()
        data = self.get_user_type_default_data()
        user = User.objects.create(**data)
        user.set_password(data['password'])
        user.save()
        self.data = {
            'username': data['username'],
            'password': data['password'],
            'account_type': data['type']
        }

    @parameterized.expand(PERMISSIONS)
    def test_can_login(self, _, user, expected):
        self.authenticate(user=user)
        response = self.login(data=self.data)
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(REQUIRED_FIELDS)
    def test_required_fields(self, _, field):
        data = self.modify_data(self.data, exclude=[field])
        response = self.login(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


@override_settings(GOOGLE_APP_ID='testappid')
class GoogleAccountLoginTestCase(AccountTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_200_OK),
        ('Authenticated User', AccountTestCase.get_user_type_default(), status.HTTP_403_FORBIDDEN),
        ('Authenticated Admin User', AccountTestCase.get_admin_user(), status.HTTP_403_FORBIDDEN)
    ]
    REQUIRED_FIELDS = [
        ('username', 'username'),
        ('access_token', 'access_token'),
        ('account_type', 'account_type')
    ]

    def setUp(self):
        super(GoogleAccountLoginTestCase, self).setUp()
        data = self.get_user_type_default_data()
        data['type'] = User.TYPE_GOOGLE
        user = User.objects.create(**data)
        user.save()
        self.data = {
            'username': data['username'],
            'access_token': data['password'],
            'account_type': data['type']
        }

    @parameterized.expand(PERMISSIONS)
    def test_can_login(self, _, user, expected):
        self.authenticate(user=user)
        response = self.login(data=self.data)
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(REQUIRED_FIELDS)
    def test_required_fields(self, _, field):
        data = self.modify_data(self.data, exclude=[field])
        response = self.login(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(GOOGLE_APP_ID=None)
    def test_config_check(self):
        response = self.login(data=self.data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST, response.data)


@override_settings(FACEBOOK_APP_ID='testappid', FACEBOOK_CLIENT_SECRET='testclientsecret')
class FacebookAccountLoginTestCase(AccountTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_200_OK),
        ('Authenticated User', AccountTestCase.get_user_type_default(), status.HTTP_403_FORBIDDEN),
        ('Authenticated Admin User', AccountTestCase.get_admin_user(), status.HTTP_403_FORBIDDEN)
    ]
    REQUIRED_FIELDS = [
        ('username', 'username'),
        ('access_token', 'access_token'),
        ('account_type', 'account_type')
    ]

    def setUp(self):
        super(FacebookAccountLoginTestCase, self).setUp()
        data = self.get_user_type_default_data()
        data['type'] = User.TYPE_FACEBOOK
        user = User.objects.create(**data)
        user.save()
        self.data = {
            'username': data['username'],
            'access_token': data['password'],
            'account_type': data['type']
        }

    @parameterized.expand(PERMISSIONS)
    def test_can_login(self, _, user, expected):
        self.authenticate(user=user)
        response = self.login(data=self.data)
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(REQUIRED_FIELDS)
    def test_required_fields(self, _, field):
        data = self.modify_data(self.data, exclude=[field])
        response = self.login(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @override_settings(FACEBOOK_APP_ID=None, FACEBOOK_CLIENT_SECRET=None)
    def test_config_check(self):
        response = self.login(data=self.data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST, response.data)


class AccountRetrieveTestCase(AccountTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_401_UNAUTHORIZED),
        ('Authenticated User', AccountTestCase.get_user_type_default(), status.HTTP_200_OK),
        ('Authenticated Admin User', AccountTestCase.get_admin_user(), status.HTTP_200_OK)
    ]

    @parameterized.expand(PERMISSIONS)
    def test_permissions(self, _, user, expected):
        self.authenticate(user=user)
        response = self.retrieve()
        self.assertEqual(response.status_code, expected)


class AccountResetPasswordRequestTestCase(AccountTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_204_NO_CONTENT),
        ('Authenticated User', AccountTestCase.get_user_type_default(), status.HTTP_403_FORBIDDEN),
        ('Authenticated Admin User', AccountTestCase.get_admin_user(), status.HTTP_403_FORBIDDEN)
    ]
    REQUIRED_FIELDS = [
        ('email', 'email')
    ]

    def setUp(self):
        super(AccountResetPasswordRequestTestCase, self).setUp()
        user = self.get_user_type_default()
        self.data = {
            'email': user.email
        }

    @parameterized.expand(PERMISSIONS)
    def test_permissions(self, _, user, expected):
        self.authenticate(user=user)
        response = self.reset(data=self.data)
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(REQUIRED_FIELDS)
    def test_required_field(self, _, field):
        data = self.modify_data(data=self.data, exclude=[field])
        response = self.reset(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_reset_password_email_sent(self):
        response = self.reset(data=self.data)
        self.assertEqual(len(mail.outbox), 1)

    def test_reset_password_validates_email(self):
        data = self.modify_data(data=self.data, injections={'email': 'notanemail@launchpeer.com'})
        response = self.reset(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_email_contains_token(self):
        self.reset(data=self.data)
        user = User.objects.get(email=self.data['email'])
        token, created = Token.objects.get_or_create(user=user)
        self.assertIn(token.key, mail.outbox[0].body)


class AccountResetPasswordTestCase(AccountTestCase):
    PERMISSIONS = [
        ('Anonymous User', None, status.HTTP_200_OK),
        ('Authenticated User', AccountTestCase.get_user_type_default(), status.HTTP_403_FORBIDDEN),
        ('Authenticated Admin User', AccountTestCase.get_admin_user(), status.HTTP_403_FORBIDDEN)
    ]
    REQUIRED_FIELDS = [
        ('token', 'token'),
        ('password', 'password')
    ]

    def setUp(self):
        super(AccountResetPasswordTestCase, self).setUp()
        user = self.get_user_type_default()
        token, created = Token.objects.get_or_create(user=user)
        self.data = {
            'token': token.key,
            'password': 'newpassword'
        }

    @parameterized.expand(PERMISSIONS)
    def test_permissions(self, _, user, expected):
        self.authenticate(user=user)
        response = self.update(data=self.data)
        self.assertEqual(response.status_code, expected)

    @parameterized.expand(REQUIRED_FIELDS)
    def test_required_field(self, _, field):
        data = self.modify_data(data=self.data, exclude=[field])
        response = self.update(data=data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)



from setuptools import setup
setup(
  name='lp_accounts',
  packages=['lp_accounts'],
  version='0.1',
  description='REST Framework for User Accounts',
  author='Jim Simon',
  author_email='hello@launchpeer.com',
  url='https://github.com/Launchpeer/django-rest-account',
  download_url='https://github.com/Launchpeer/django-rest-account/archive/master.tar.gz',
  keywords=[],
  classifiers=[],
)


from rest_framework import permissions


class IsAnonUser(permissions.IsAuthenticated):
    def has_permission(self, request, view):
        return not super(IsAnonUser, self).has_permission(request, view)
